@@ -18,9 +18,13 @@ if( margin == "probit" ){
   
   }
   
-  if(bc == TRUE){ # RECALL that this is deriv of Y = 0 wrt eta1 and not the usual case of Y = 1!!
-  derp1.dereta1      <- -dnorm(-eta)         # First derivative of 1-prob(eta1) respect to eta1.
-  der2p1.dereta1eta1 <- eta * dnorm(-eta)  ## This is the second derivative of 1 - p1 respect to eta1  
+  if(bc == TRUE){ 
+  
+  # RECALL that this is deriv of Y = 0 wrt eta1 and not the usual case of Y = 1!!
+  derp1.dereta1      <- -dnorm(-eta)         # First derivative of 1-prob(eta1) respect to eta1
+  
+  der2p1.dereta1eta1 <- eta * dnorm(-eta)  ## This is the second derivative of 1 - p1 respect to eta1 
+  
   }  
     
     
@@ -45,21 +49,15 @@ if( margin == "logit" ){
       
   der2p1.dereta1eta1 <- (1 - (3 - 2 * (exp(-eta)/(1 + exp(-eta)))) * exp(-eta)/(1 + exp(-eta))) * 
                            exp(-eta)/(1 + exp(-eta))
- ## This is the second derivative of 1 - p1 respect to eta1  
+
   }
   
 }
 
 
 
-#if( margin == "loglog" ){
-#
-#  pr  <- exp(-exp(eta)) 
-#  d.n <- -(exp(-exp(eta)) * exp(eta))
-#  d.n <- ifelse(d.n < epsilon, epsilon, d.n )
-#  der2p.dereta <- -((1 - exp(eta)) * exp(-exp(eta)) * exp(eta)) 
-#    
-#} #not in glm
+
+
 
 if( margin == "cloglog" ){
  
@@ -86,6 +84,9 @@ if( margin == "cloglog" ){
  
 }
 
+
+
+
 if( margin == "cauchit" ){
  
   pr  <- 1 / pi * atan(eta) + 0.5
@@ -105,7 +106,7 @@ if( margin == "cauchit" ){
   derp1.dereta1    <- -(1/(pi * (1 + eta^2)))
          
   der2p1.dereta1eta1 <- 2 * (eta/(pi * (1 + eta^2)^2))
- ## This is the second derivative of 1 - p1 respect to eta1 
+
 
   }
   
@@ -113,6 +114,47 @@ if( margin == "cauchit" ){
 }
 
 
+
+
+
+
+
+
+if( margin == "log" ){
+ 
+  pr  <- exp(eta)
+  
+  
+  if(only.pr == FALSE){
+  
+  d.n <- exp(eta)
+  d.n <- ifelse(d.n < epsilon, epsilon, d.n )
+  der2p.dereta <- exp(eta)
+  
+  
+  }
+  
+  
+  if(bc == TRUE){
+  
+  derp1.dereta1    <- -exp(eta)  
+                   
+  der2p1.dereta1eta1 <- -exp(eta)
+ 
+                }
+ 
+}
+
+
+
+
+
+
+
+
+
+
+
   pr <- mm(pr) 
   
     
